import datetime as dt
import numpy as np
import pandas as pd


class Quant(object):
    @staticmethod
    def get_daily_returns(values: pd.Series) -> pd.Series:
        """Compute and return the daily return values."""
        return values.pct_change(1).fillna(0)

    @staticmethod
    def get_CAGR(values: pd.Series, traded_days: int = 252) -> pd.Series:
        """Calculates the Compound Annual Growth Rate."""
        return ((1 + values.pct_change()).cumprod()) ** (traded_days / len(values)) - 1

    @staticmethod
    def get_volatility(values: pd.Series, traded_days: int = 252) -> pd.Series:
        """Calculates the daily Volatility over a year, given a number of traded days."""
        return values.pct_change().std() * np.sqrt(traded_days)

    @staticmethod
    def get_sharpe(values: pd.Series, traded_days: int = 252, rf: pd.Series = None) -> pd.Series:
        """Sharpe ratio. RF = risk free rate. If not specified then rf=zero"""
        if not rf:
            rf = np.array([0] * len(values))

        return (Quant.get_CAGR(values, traded_days) - rf) / Quant.get_volatility(values, traded_days)

    @staticmethod
    def get_rolling_mean(values: pd.Series, window: int = 20) -> pd.Series:
        """Return rolling mean of given values, using specified window size."""
        return values.rolling(window=window, center=False).mean()

    @staticmethod
    def get_rolling_std(values: pd.Series, window: int = 20) -> pd.Series:
        """Return rolling standard deviation of given values, using specified window size."""
        return values.rolling(window=window, center=False).std()

    @staticmethod
    def get_bollinger_bands(rm: pd.Series, rstd: pd.Series) -> pd.Series:
        """Return upper and lower Bollinger Bands."""
        upper_band = rm + (rstd * 2)
        lower_band = rm - (rstd * 2)

        return upper_band, lower_band



import sys
sys.path.append("..")
from exploratium.quant import Quant as q
from typing import List, Tuple
import datetime as dt
import pandas as pd
import pandas_datareader as pdr


def load_ticker_data(
    tickers: List[str],
    start: str = (dt.datetime.today() + dt.timedelta(-365)).isoformat(),
    end: str = dt.datetime.today().isoformat(),
) -> List[Tuple[str, pd.DataFrame]]:
    """Saves current enriched dataframes to disk.

    Keyword arguments:
    tickers -- the symbols to save (default)
    """

    ticker_data = []

    for ticker in tickers:
        ticker_data.append((ticker, pdr.get_data_yahoo(ticker, start, end)))

    return ticker_data


def transform_ticker_data(
    tickers: List[pd.DataFrame],
    out_dir: str = "./data",
    traded_days: int = 252,
    field: str = "Adj Close"
) -> List[str]:
    """Saves current enriched dataframes to disk.

    Keyword arguments:
    tickers -- the symbols to save (default)
    """

    ticker_files = []

    for ticker, df in tickers:
        df["ma7"] = q.get_rolling_mean(df[field], 7)
        df["ma30"] = q.get_rolling_mean(df[field], 30)
        df["ma90"] = q.get_rolling_mean(df[field], 90)

        df["upper_bollinger_band"], df["lower_bollinger_band"] = q.get_bollinger_bands(
            q.get_rolling_mean(df[field]), q.get_rolling_std(
                df[field])
        )

        df["daily_returns"] = q.get_daily_returns(df[field])
        df["sharpe"] = q.get_sharpe(df[field], traded_days)

        path = "{}/{}.csv".format(out_dir, ticker)
        df.to_csv(path)
        ticker_files.append(path)

    return ticker_files



import setuptools
from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="exploratium-quant",  # Replace with your own username
    version="0.0.3",
    author="Carlos Rivera",
    author_email="carlos@synx.ai",
    description="ETL library to download and pre-process financial data.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/exploratium/quant-cli",
    project_urls={
        "Bug Tracker": "https://github.com/exploratium/quant-cli/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "./"},
    packages=setuptools.find_packages(where="./"),
    python_requires=">=3.6",
    py_modules=['exploratium', 'utils'],
    install_requires=[
        'click',
        'blessed',
        'pandas-datareader'
    ],
    scripts=['bin/quant-cli'],
)
